@@ -51,6 +51,7 @@ struct _PanelDItemEditorPrivate
 	guint     save_timeout;
 
 	char     *uri; /* file location */
+	gboolean  type_directory;
 	gboolean  new_file;
 	gboolean  combo_setuped;
 
@@ -158,6 +159,7 @@ enum {
 	PROP_0,
 	PROP_KEYFILE,
 	PROP_URI,
+	PROP_TYPEDIRECTORY
 };
 
 static guint ditem_edit_signals[LAST_SIGNAL] = { 0 };
@@ -188,6 +190,10 @@ static gboolean panel_ditem_editor_load_uri (PanelDItemEditor  *dialog,
 static void panel_ditem_editor_set_key_file (PanelDItemEditor *dialog,
 					     GKeyFile         *key_file);
 
+static gboolean panel_ditem_editor_get_type_directory (PanelDItemEditor *dialog);
+static void panel_ditem_editor_set_type_directory (PanelDItemEditor *dialog,
+						   gboolean          type_directory);
+
 
 static PanelDItemEditorType
 map_type_from_desktop_item (const char *type,
@@ -217,6 +223,7 @@ panel_ditem_editor_constructor (GType                  type,
 	PanelDItemEditor *dialog;
 	GnomeVFSURI      *vfs_uri;
 	gboolean          loaded;
+	char             *desktop_type;
 
 	obj = G_OBJECT_CLASS (panel_ditem_editor_parent_class)->constructor (type,
 									     n_construct_properties,
@@ -231,6 +238,9 @@ panel_ditem_editor_constructor (GType                  type,
 		loaded = TRUE;
 	} else {
 		dialog->priv->key_file = panel_util_key_file_new_desktop ();
+		if (dialog->priv->type_directory)
+			panel_util_key_file_set_string (dialog->priv->key_file,
+							"Type", "Directory");
 		dialog->priv->free_key_file = TRUE;
 		loaded = FALSE;
 	}
@@ -251,6 +261,12 @@ panel_ditem_editor_constructor (GType                  type,
 
 	dialog->priv->dirty = FALSE;
 
+	desktop_type = panel_util_key_file_get_string (dialog->priv->key_file,
+						       "Type");
+	if (desktop_type && !strcmp (desktop_type, "Directory"))
+		dialog->priv->type_directory = TRUE;
+	g_free (desktop_type);
+
 	panel_ditem_editor_setup_ui (dialog);
 
 	if (dialog->priv->new_file)
@@ -278,6 +294,9 @@ panel_ditem_editor_get_property (GObject    *object,
 	case PROP_URI:
 		g_value_set_string (value, panel_ditem_editor_get_uri (dialog));
 		break;
+	case PROP_TYPEDIRECTORY:
+		g_value_set_boolean (value, panel_ditem_editor_get_type_directory (dialog));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -298,10 +317,16 @@ panel_ditem_editor_set_property (GObject       *object,
 
 	switch (prop_id) {
 	case PROP_KEYFILE:
-		panel_ditem_editor_set_key_file (dialog, g_value_get_pointer (value));
+		panel_ditem_editor_set_key_file (dialog,
+						 g_value_get_pointer (value));
 		break;
 	case PROP_URI:
-		panel_ditem_editor_set_uri (dialog, g_value_get_string (value));
+		panel_ditem_editor_set_uri (dialog,
+					    g_value_get_string (value));
+		break;
+	case PROP_TYPEDIRECTORY:
+		panel_ditem_editor_set_type_directory (dialog,
+						       g_value_get_boolean (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -458,6 +483,15 @@ panel_ditem_editor_class_init (PanelDItemEditorClass *class)
 				     "The URI of the .desktop file",
 				     NULL,
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+	g_object_class_install_property (
+		gobject_class,
+		PROP_TYPEDIRECTORY,
+		g_param_spec_boolean ("type-directory",
+				      "Type Directory",
+				      "Whether the edited file is a .directory file or not",
+				      FALSE,
+				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static GtkWidget *
@@ -549,6 +583,9 @@ panel_ditem_editor_get_item_type (PanelDItemEditor *dialog)
 	GtkTreeModel         *model;
 	PanelDItemEditorType  type;
 
+	if (dialog->priv->type_directory)
+		return PANEL_DITEM_EDITOR_TYPE_DIRECTORY;
+
 	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (dialog->priv->type_combo),
 					    &iter))
 		return PANEL_DITEM_EDITOR_TYPE_NULL;
@@ -663,6 +700,8 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 		gtk_widget_hide (priv->close_button);
 		gtk_widget_show (priv->cancel_button);
 		gtk_widget_show (priv->ok_button);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+						 GTK_RESPONSE_OK);
 
 		if (!priv->combo_setuped) {
 			setup_combo (priv->type_combo,
@@ -673,13 +712,15 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 
 		gtk_combo_box_set_active (GTK_COMBO_BOX (priv->type_combo), 0);
 
-		show_combo = TRUE;
+		show_combo = !priv->type_directory;
 	} else {
 
 		gtk_widget_show (priv->revert_button);
 		gtk_widget_show (priv->close_button);
 		gtk_widget_hide (priv->cancel_button);
 		gtk_widget_hide (priv->ok_button);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+						 GTK_RESPONSE_CLOSE);
 
 		show_combo = (type != PANEL_DITEM_EDITOR_TYPE_LINK) &&
 			     (type != PANEL_DITEM_EDITOR_TYPE_DIRECTORY);
@@ -723,8 +764,6 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 				break;
 			}
 		} while (gtk_tree_model_iter_next (model, &iter));
-
-		gtk_widget_grab_focus (priv->type_combo);
 	} else if (type == PANEL_DITEM_EDITOR_TYPE_DIRECTORY) {
 		table_attach_label (GTK_TABLE (priv->table), priv->name_label,
 				    1, 2, 0, 1);
@@ -735,8 +774,6 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 				    1, 2, 1, 2);
 		table_attach_entry (GTK_TABLE (priv->table), priv->comment_entry,
 				    2, 3, 1, 2);
-
-		gtk_widget_grab_focus (priv->name_entry);
 	} else {
 		table_attach_label (GTK_TABLE (priv->table), priv->name_label,
 				    1, 2, 0, 1);
@@ -752,8 +789,6 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 				    1, 2, 2, 3);
 		table_attach_entry (GTK_TABLE (priv->table), priv->comment_entry,
 				    2, 3, 2, 3);
-
-		gtk_widget_grab_focus (priv->name_entry);
 	}
 
 	type_combo_changed (dialog);
@@ -770,6 +805,8 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 	gtk_container_set_focus_chain (GTK_CONTAINER (priv->table),
 				       focus_chain);
 	g_list_free (focus_chain);
+
+	gtk_widget_grab_focus (priv->name_entry);
 }
 
 /*
@@ -799,6 +836,17 @@ panel_ditem_editor_changed (PanelDItemEditor *dialog)
 	g_signal_emit (G_OBJECT (dialog), ditem_edit_signals[CHANGED], 0);
 }
 
+static void
+panel_ditem_editor_activated (PanelDItemEditor *dialog)
+{
+	if (GTK_WIDGET_VISIBLE (dialog->priv->ok_button))
+		gtk_dialog_response (GTK_DIALOG (dialog),
+				     GTK_RESPONSE_OK);
+	else if (GTK_WIDGET_VISIBLE (dialog->priv->close_button))
+		gtk_dialog_response (GTK_DIALOG (dialog),
+				     GTK_RESPONSE_CLOSE);
+}
+
 static void
 panel_ditem_editor_name_changed (PanelDItemEditor *dialog)
 {
@@ -810,8 +858,8 @@ panel_ditem_editor_name_changed (PanelDItemEditor *dialog)
 		panel_util_key_file_set_locale_string (dialog->priv->key_file,
 						       "Name", name);
 	else
-		panel_util_key_file_remove_locale_key (dialog->priv->key_file,
-						       "Name");
+		panel_util_key_file_remove_all_locale_key (dialog->priv->key_file,
+							   "Name");
 
 	g_signal_emit (G_OBJECT (dialog), ditem_edit_signals[NAME_CHANGED], 0,
 		       name);
@@ -880,8 +928,8 @@ panel_ditem_editor_comment_changed (PanelDItemEditor *dialog)
 		panel_util_key_file_set_locale_string (dialog->priv->key_file,
 						       "Comment", comment);
 	else
-		panel_util_key_file_remove_locale_key (dialog->priv->key_file,
-						       "Comment");
+		panel_util_key_file_remove_all_locale_key (dialog->priv->key_file,
+							   "Comment");
 
 	g_signal_emit (G_OBJECT (dialog), ditem_edit_signals[COMMENT_CHANGED],
 		       0, comment);
@@ -916,8 +964,8 @@ panel_ditem_editor_icon_changed (PanelDItemEditor *dialog)
 		panel_util_key_file_set_locale_string (dialog->priv->key_file,
 						       "Icon", icon);
 	else
-		panel_util_key_file_remove_locale_key (dialog->priv->key_file,
-						       "Icon");
+		panel_util_key_file_remove_all_locale_key (dialog->priv->key_file,
+							   "Icon");
 
 	g_signal_emit (G_OBJECT (dialog), ditem_edit_signals[ICON_CHANGED], 0,
 		       file);
@@ -1039,6 +1087,16 @@ panel_ditem_editor_connect_signals (PanelDItemEditor *dialog)
 	CONNECT_CHANGED (priv->comment_entry, panel_ditem_editor_comment_changed);
 	CONNECT_CHANGED (priv->icon_entry, panel_ditem_editor_icon_changed);
 
+	g_signal_connect_swapped (priv->name_entry, "activate",
+				  G_CALLBACK (panel_ditem_editor_activated),
+				  dialog);
+	g_signal_connect_swapped (priv->command_entry, "activate",
+				  G_CALLBACK (panel_ditem_editor_activated),
+				  dialog);
+	g_signal_connect_swapped (priv->comment_entry, "activate",
+				  G_CALLBACK (panel_ditem_editor_activated),
+				  dialog);
+
 	g_signal_connect_swapped (priv->command_browse_button, "clicked",
 				  G_CALLBACK (command_browse_button_clicked),
 				  dialog);
@@ -1068,6 +1126,7 @@ panel_ditem_editor_init (PanelDItemEditor *dialog)
 	priv->dirty = FALSE;
 	priv->save_timeout = 0;
 	priv->uri = NULL;
+	priv->type_directory = FALSE;
 	priv->new_file = TRUE;
 	priv->save_uri = NULL;
 	priv->save_uri_data = NULL;
@@ -1142,7 +1201,7 @@ setup_icon_entry (PanelDItemEditor *dialog,
 	char *tmpstr;
 
 	if (!icon_name || icon_name[0] == '\0') {
-		if (panel_ditem_editor_get_item_type (dialog) == PANEL_DITEM_EDITOR_TYPE_DIRECTORY) {
+		if (dialog->priv->type_directory) {
 			buffer = g_strdup (PANEL_ICON_FOLDER);
 		} else {
 			buffer = g_strdup (PANEL_ICON_LAUNCHER);
@@ -1262,10 +1321,9 @@ static gboolean
 panel_ditem_editor_save (PanelDItemEditor *dialog,
 			 gboolean          report_errors)
 {
-	PanelDItemEditorType  type;
-	GKeyFile             *key_file;
-	const char           *const_buf;
-	GError               *error;
+	GKeyFile   *key_file;
+	const char *const_buf;
+	GError     *error;
 
 	g_return_val_if_fail (dialog != NULL, FALSE);
 	g_return_val_if_fail (dialog->priv->save_uri != NULL ||
@@ -1281,19 +1339,28 @@ panel_ditem_editor_save (PanelDItemEditor *dialog,
 	/* Verify that the required informations are set */
 	const_buf = gtk_entry_get_text (GTK_ENTRY (dialog->priv->name_entry));
 	if (const_buf == NULL || const_buf [0] == '\0') {
-		if (report_errors)
-			g_signal_emit (G_OBJECT (dialog),
-				       ditem_edit_signals[ERROR_REPORTED], 0,
-				       _("Could not save launcher"),
-				       _("The name of the launcher is not set."));
+		if (report_errors) {
+			if (!dialog->priv->type_directory)
+				g_signal_emit (G_OBJECT (dialog),
+					       ditem_edit_signals[ERROR_REPORTED], 0,
+					       _("Could not save launcher"),
+					       _("The name of the launcher is not set."));
+			else
+				g_signal_emit (G_OBJECT (dialog),
+					       ditem_edit_signals[ERROR_REPORTED], 0,
+					       _("Could not save directory properties"),
+					       _("The name of the directory is not set."));
+		}
 		return FALSE;
 	}
 
-	type = panel_ditem_editor_get_item_type (dialog);
 	const_buf = gtk_entry_get_text (GTK_ENTRY (dialog->priv->command_entry));
-	if (type != PANEL_DITEM_EDITOR_TYPE_DIRECTORY &&
+	if (!dialog->priv->type_directory &&
 	    (const_buf == NULL || const_buf [0] == '\0')) {
-		char *error;
+		PanelDItemEditorType  type;
+		char                 *error;
+
+		type = panel_ditem_editor_get_item_type (dialog);
 
 		switch (type) {
 		case PANEL_DITEM_EDITOR_TYPE_APPLICATION:
@@ -1435,7 +1502,7 @@ panel_ditem_editor_revert (PanelDItemEditor *dialog)
 						revert_key_file,
 						revert_keys [i].key);
 				if (string == NULL)
-					panel_util_key_file_remove_locale_key (
+					panel_util_key_file_remove_all_locale_key (
 							key_file,
 							revert_keys [i].key);
 				else
@@ -1579,11 +1646,12 @@ panel_ditem_editor_load_uri (PanelDItemEditor  *dialog,
 	return TRUE;
 }
 
-GtkWidget *
-panel_ditem_editor_new (GtkWindow   *parent,
-			GKeyFile    *key_file,
-			const char  *uri,
-			const char  *title)
+static GtkWidget *
+panel_ditem_editor_new_full (GtkWindow   *parent,
+			     GKeyFile    *key_file,
+			     const char  *uri,
+			     const char  *title,
+			     gboolean     type_directory)
 {
 	GtkWidget *dialog;
 
@@ -1591,6 +1659,7 @@ panel_ditem_editor_new (GtkWindow   *parent,
 			       "title", title,
 			       "keyfile", key_file,
 			       "uri", uri,
+			       "type-directory", type_directory,
 			       NULL);
 
 	if (parent)
@@ -1599,6 +1668,26 @@ panel_ditem_editor_new (GtkWindow   *parent,
 	return dialog;
 }
 
+GtkWidget *
+panel_ditem_editor_new (GtkWindow   *parent,
+			GKeyFile    *key_file,
+			const char  *uri,
+			const char  *title)
+{
+	return panel_ditem_editor_new_full (parent, key_file, uri,
+					    title, FALSE);
+}
+
+GtkWidget *
+panel_ditem_editor_new_directory (GtkWindow   *parent,
+				  GKeyFile    *key_file,
+				  const char  *uri,
+				  const char  *title)
+{
+	return panel_ditem_editor_new_full (parent, key_file, uri,
+					    title, TRUE);
+}
+
 static void
 panel_ditem_editor_set_key_file (PanelDItemEditor *dialog,
 				 GKeyFile         *key_file)
@@ -1638,6 +1727,20 @@ panel_ditem_editor_set_uri (PanelDItemEditor *dialog,
 	g_object_notify (G_OBJECT (dialog), "uri");
 }
 
+static void
+panel_ditem_editor_set_type_directory (PanelDItemEditor *dialog,
+				       gboolean          type_directory)
+{
+	g_return_if_fail (PANEL_IS_DITEM_EDITOR (dialog));
+
+	if (dialog->priv->type_directory == type_directory)
+		return;
+
+	dialog->priv->type_directory = type_directory;
+
+	g_object_notify (G_OBJECT (dialog), "type-directory");
+}
+
 GKeyFile *
 panel_ditem_editor_get_key_file (PanelDItemEditor *dialog)
 {
@@ -1662,6 +1765,14 @@ panel_ditem_editor_get_uri (PanelDItemEditor *dialog)
 	return dialog->priv->uri;
 }
 
+static gboolean
+panel_ditem_editor_get_type_directory (PanelDItemEditor *dialog)
+{
+	g_return_val_if_fail (PANEL_IS_DITEM_EDITOR (dialog), FALSE);
+
+	return dialog->priv->type_directory;
+}
+
 void
 panel_ditem_register_save_uri_func (PanelDItemEditor  *dialog,
 				    PanelDitemSaveUri  save_uri,
